import util from './util/index'
import WebsocketHandle from './lib/WebsocketHandle'

class Runtime extends WebsocketHandle {
  constructor (websocket, runtime) {
    super(websocket)
    this.runtime = runtime
    this.enable = false
    this.runtimeHandlers = {
      'Runtime.enable': this.handleEnableCommand,
      'Runtime.getIsolateId': this.handleGetIsolateId,
      'Runtime.runIfWaitingForDebugger': this.handleRunIfWaitingForDebugger,
      'Runtime.getProperties': this.handleGetProperties,
      'Runtime.releaseObjectGroup': this.handleReleaseObjectGroup,
      'Runtime.compileScript': this.handleCompileScript,
      'Runtime.evaluate': this.handleEvaluate
    }
  }

  handle (e) {
    const t = this.runtimeHandlers[e.method]
    t ? t.call(this, e.id, e.params) : this.respond(e.id)
  }

  handleEnableCommand (e) {
    this.fireExecutionContextCreatedEvent()
    this.respond(e)
    this.enable = true
  }

  handleGetIsolateId (e) {
    this.respond(e, {
      id: '0'
    })
  }

  handleRunIfWaitingForDebugger (e) {
    this.respond(e)
    // this.runtime.launch()
  }

  handleGetProperties (id, params) {
    const res = this.runtime.getItem(params.objectId)
    this.respond(id, {
      result: util.EvaluatePropertiesRes(res, this.runtime)
    })
  }

  handleReleaseObjectGroup (e) {
    // var r = t.objectGroup;
    // this.runtime.releaseObjectGroup(r),
    this.respond(e, {})
  }

  handleCompileScript (e) {
    this.respond(e, {})
  }

  handleEvaluate (id, t) {
    if (t.objectGroup === 'console') {
      const obj = util.evalScript(t.expression)
      let result = {}
      if (typeof obj === 'object') {
        const itemId = this.runtime.setItem(obj)
        obj.__objectId__ = `${itemId}_0`
        this.runtime.setObjectId(obj, itemId)
        result = util.EvaluateRes(obj, [], null, this.runtime)
      } else {
        result = util.EvaluateRes(obj, [], null, this.runtime)
      }
      this.respond(id, {
        result: {
          result: result
        }
      })
    }
  }

  fireExecutionContextCreatedEvent () {
    const e = {
      context: {
        id: 1,
        // id: this.runtime.getContextId(),
        origin: 'http://49.232.147.237',
        name: 'sub_context'
      }
    }
    this.fireEvent('Runtime.executionContextCreated', e)
  }

  onConsoleOutput (data) {
    try {
      const arg = util.getArgByArguments(data, this.runtime)
      const u = {
        type: 'log',
        args: arg,
        executionContextId: 0,
        timestamp: Date.now(),
        stackTrace: { callFrames: {} }
      }
      this.fireEvent('Runtime.consoleAPICalled', u)
    } catch (error) {
    }
  }
}

export default Runtime
