import Runtime from './Runtime'
import Debugger from './Debugger'
import RuntimeIns from './lib/RuntimeIns'
import DebuggerIns from './lib/DebuggerIns'
import timer from './util/timer'

const rins = new RuntimeIns()
const dins = new DebuggerIns()

let runtime = null

$window.finclip_debug_warn_log = function () {
  console.warn('请优化代码结构, 便于调试')
}

$window.finclip_debug_before = function (data, arg) {
  dins.debuggerObj.oldData = data
}

$window.finclip_debug_after = function (scriptId, line, funName) {
  const { oldData, breakpoint, callFrame } = dins.debuggerObj
  // if (debuggerObj.nowScriptId !== scriptId) {
  //   debuggerObj.callFrame = []
  // }
  const item = breakpoint.find((e) => {
    return e.scriptId == scriptId && line === e.line
  })
  if (funName.name !== '') {
    dins.debuggerObj.callFrame.push(funName.name)
  }
  if (item) {
    debuggerins.debuggerFirePausedEvent({
      res: oldData,
      callFrame: callFrame,
      t: item.start
    })
  }
  dins.debuggerObj.nowScriptId = scriptId
}

// [基础库 -> sdk] -> remote
const sendMsg = (target, payload) => {
  $window.remoteDebugger && $window.remoteDebugger.remoteDebugger({
    target,
    data: payload
  })
}

const ws = {
  send: function (msg) {
    sendMsg('runtime', msg)
  }
}

runtime = new Runtime(ws, rins, dins)
timer.init(runtime)
let debuggerins

const handleRuntimeCDP = (payload) => {
  const params = {
    id: payload.id,
    method: payload.method || '',
    params: payload.params
  }
  if (params.method.startsWith('Runtime.')) {
    runtime.handle(params)
  }
  if (params.method.startsWith('Debugger.')) {
    if (!debuggerins) debuggerins = new Debugger(ws, rins, dins)
    debuggerins.handle(params)
  }
}

let currentPage
setTimeout(() => {
  $window.execOnServiceReady(() => {
    wx.onAppRoute(function (params) {
      currentPage = params
      sendMsg('dom', { method: 'DOM.documentUpdated' })
    })
  })
}, 0)

const handleDomCDP = (payload) => {
  // Dom
  $window.ServiceJSBridge.publish('remoteDebugger', payload, [currentPage.webviewId])
}

// sdk -> 基础库
$window.ServiceJSBridge.on('remoteDebugger', (msg) => {
  if (msg.target === 'runtime') {
    handleRuntimeCDP(msg.data)
    return
  }
  if (msg.target === 'dom') {
    handleDomCDP(msg.data)
    return
  }
  if (msg.data.id) {
    sendMsg(msg.target, {
      id: msg.data.id,
      result: {}
    })
  }
})

$window.iosRemoteDebuggerLog = function iosRemoteDebuggerLog (data) {
  if (runtime && runtime.enable && !$window.isIDE) {
    runtime.onConsoleOutput(data)
  }
}

function testDebug () {
  setTimeout(() => {
    function aaa () {
      runtime.onConsoleOutput(arguments)
    }
    const mapData = new Map()
    mapData.set(1, 2)
    const mySet = new Set()
    mySet.add(1)
    mySet.add({
      c: 1
    })
    aaa({
      a: 1,
      b: 2
    }, { c: 3 }, 123)
    aaa({
      a: 1,
      b: 2
    }, { c: 3 })
    aaa({
      a: 1,
      b: 2
    })
    aaa({ ff: 2, dd: { a: 3 }, c: mapData }, 1)
    aaa(mySet, mapData, '==========')
    aaa(mySet)
    aaa('======')
    aaa(1, 2, 3, 4)
  }, 5000 * 2)
}

// testDebug()
